import { createReducer, createSelector, } from '@ngrx/store';
export function createTraitFactory(f) {
    return f;
}
export function createEntityFeatureFactory(...traits) {
    return ((config) => {
        const sortedTraits = sortTraits([...traits]);
        const allConfigs = sortedTraits.reduce((acc, factory) => {
            acc[factory.key] = factory.config;
            return acc;
        }, {});
        const allActions = sortedTraits.reduce((previousResult, factory) => {
            var _a, _b;
            let result = (_b = (_a = factory === null || factory === void 0 ? void 0 : factory.actions) === null || _a === void 0 ? void 0 : _a.call(factory, {
                actionsGroupKey: config.actionsGroupKey,
                allConfigs,
            })) !== null && _b !== void 0 ? _b : {};
            result = previousResult ? Object.assign(Object.assign({}, previousResult), result) : result;
            return result;
        }, {});
        const allSelectors = sortedTraits.reduce((previousResult, factory) => {
            var _a, _b;
            let result = (_b = (_a = factory === null || factory === void 0 ? void 0 : factory.selectors) === null || _a === void 0 ? void 0 : _a.call(factory, {
                previousSelectors: previousResult,
                allConfigs,
            })) !== null && _b !== void 0 ? _b : {};
            result = previousResult ? Object.assign(Object.assign({}, previousResult), result) : result;
            return result;
        }, {});
        const allMutators = sortedTraits.reduce((previousResult, factory) => {
            var _a, _b;
            let result = (_b = (_a = factory === null || factory === void 0 ? void 0 : factory.mutators) === null || _a === void 0 ? void 0 : _a.call(factory, {
                allSelectors: allSelectors,
                previousMutators: previousResult,
                allConfigs,
            })) !== null && _b !== void 0 ? _b : {};
            result = previousResult ? Object.assign(Object.assign({}, previousResult), result) : result;
            return result;
        }, {});
        const initialState = sortedTraits.reduce((previousResult, factory) => {
            var _a, _b, _c;
            const result = (_c = (_b = (_a = factory === null || factory === void 0 ? void 0 : factory.initialState) === null || _a === void 0 ? void 0 : _a.call(factory, {
                previousInitialState: previousResult,
                allConfigs,
            })) !== null && _b !== void 0 ? _b : previousResult) !== null && _c !== void 0 ? _c : {};
            return result;
        }, {});
        const reducer = sortedTraits.reduce((previousResult, factory) => {
            var _a;
            const result = (_a = factory === null || factory === void 0 ? void 0 : factory.reducer) === null || _a === void 0 ? void 0 : _a.call(factory, {
                initialState,
                allActions,
                allSelectors,
                allMutators,
                allConfigs,
            });
            return result && previousResult
                ? (state = initialState, action) => {
                    const aState = previousResult(state, action);
                    return result(aState, action);
                }
                : result !== null && result !== void 0 ? result : previousResult;
        }, undefined);
        const allFeatureSelectors = allSelectors &&
            getSelectorsForFeature(config.featureSelector, allSelectors);
        const allEffects = sortedTraits.reduce((previousResult, factory) => {
            var _a, _b;
            let result = (_b = (_a = factory === null || factory === void 0 ? void 0 : factory.effects) === null || _a === void 0 ? void 0 : _a.call(factory, {
                allActions,
                allSelectors: allFeatureSelectors,
                allConfigs,
            })) !== null && _b !== void 0 ? _b : [];
            result = previousResult ? [...previousResult, ...result] : result;
            return result;
        }, []);
        return {
            actions: allActions,
            selectors: allFeatureSelectors,
            mutators: allMutators,
            initialState,
            reducer: reducer !== null && reducer !== void 0 ? reducer : createReducer(initialState),
            effects: allEffects,
        };
    });
}
function sortTraits(traits) {
    var _a;
    const sortedTraits = [];
    for (let i = 0; i < traits.length; i++) {
        const trait = traits[i];
        if (!((_a = trait.depends) === null || _a === void 0 ? void 0 : _a.length)) {
            sortedTraits.push(trait);
            continue;
        }
        if (trait.depends.length > 1)
            for (const d of trait.depends) {
                const isTraitPresent = traits.some((tr) => tr.key === d);
                if (isTraitPresent) {
                    trait.depends = [d];
                    break;
                }
            }
        if (trait.depends.length > 1)
            throw Error('could not find dependencies ' + trait.depends.join(' '));
        const isDependencyAlreadyAdded = sortedTraits.some((tr) => { var _a; return tr.key === ((_a = trait === null || trait === void 0 ? void 0 : trait.depends) === null || _a === void 0 ? void 0 : _a[0]); });
        if (isDependencyAlreadyAdded)
            sortedTraits.push(trait);
        else {
            // move trait to the end
            delete traits[i];
            traits.push(trait);
        }
    }
    return sortedTraits;
}
function getSelectorsForFeature(featureSelect, selectors) {
    const ss = {};
    for (const prop in selectors) {
        ss[prop] = createSelector(featureSelect, selectors[prop]);
    }
    return ss;
}
export function setPropertyReducer(sourceReducer, property, propertyReducer) {
    return function reducer(state, action) {
        const sourceState = sourceReducer(state, action);
        return Object.assign(Object.assign({}, sourceState), { [property]: propertyReducer(sourceState[property], action) });
    };
}
export function setPropertiesReducer(sourceReducer, propertiesReducers) {
    return function reducer(state, action) {
        const newState = Object.assign({}, sourceReducer(state, action));
        for (const property in propertiesReducers) {
            newState[property] = propertiesReducers[property](newState[property], action);
        }
        return newState;
    };
}
export function joinReducers(firstReducer, secondReducer) {
    return function reducer(state, action) {
        const sourceState = firstReducer(state, action);
        return secondReducer(sourceState, action);
    };
}
//# sourceMappingURL=create-entity-feature.js.map