import { createSelector } from '@ngrx/store';
export function createPaginationTraitSelectors(previousSelectors, allConfigs) {
    var _a;
    const { selectAll, isLoading } = previousSelectors;
    const filterFunction = (_a = allConfigs === null || allConfigs === void 0 ? void 0 : allConfigs.filter) === null || _a === void 0 ? void 0 : _a.filterFn;
    function selectPagination(state) {
        return state.pagination;
    }
    const selectPaginationFiltered = filterFunction
        ? createSelector(selectAll, selectPagination, (entities, pagination) => {
            return Object.assign(Object.assign({}, pagination), { total: entities.length, cache: Object.assign(Object.assign({}, pagination.cache), { start: 0, end: entities.length }) });
        })
        : selectPagination;
    const selectPageEntities = createSelector(selectAll, selectPaginationFiltered, (entities, pagination, { page } = { page: pagination.currentPage }) => {
        const startIndex = page * pagination.pageSize - pagination.cache.start;
        let endIndex = startIndex + pagination.pageSize;
        endIndex =
            endIndex < pagination.cache.end ? endIndex : pagination.cache.end;
        return entities.slice(startIndex, endIndex);
    });
    const selectPageInfo = createSelector(selectPaginationFiltered, (pagination) => {
        const pagesCount = pagination.total && pagination.total > 0
            ? Math.ceil(pagination.total / pagination.pageSize)
            : undefined;
        return {
            pageIndex: pagination.currentPage,
            total: pagination.total,
            pageSize: pagination.pageSize,
            pagesCount,
            hasPrevious: pagination.currentPage - 1 >= 0,
            hasNext: pagination.total && pagination.total > 0
                ? pagination.currentPage + 1 < pagesCount
                : true,
            cacheType: pagination.cache.type,
        };
    });
    const isPageInCache = createSelector(selectPaginationFiltered, (pagination, { page } = { page: pagination.currentPage }) => {
        const startIndex = page * pagination.pageSize;
        let endIndex = startIndex + pagination.pageSize - 1;
        endIndex =
            pagination.total && endIndex > pagination.total
                ? pagination.total - 1
                : endIndex;
        return (startIndex >= pagination.cache.start && endIndex <= pagination.cache.end);
    });
    const selectPage = createSelector(selectPageEntities, selectPageInfo, 
    // props look unsued but they are pass to the selectPageEntities
    // eslint-disable-next-line @typescript-eslint/no-unused-vars
    (entities, pageInfo, props = { page: pageInfo.pageIndex }) => (Object.assign({ entities }, pageInfo)));
    const selectPagedRequest = createSelector(selectPagination, (pagination) => ({
        startIndex: pagination.pageSize * pagination.requestPage,
        size: pagination.pageSize * pagination.pagesToCache,
        page: pagination.requestPage,
    }));
    const isLoadingPage = createSelector(isLoading, selectPagination, (isLoading, pagination) => isLoading && pagination.requestPage === pagination.currentPage);
    return {
        selectPageEntities,
        isPageInCache,
        selectPage,
        selectPagedRequest,
        selectPageInfo,
        isLoadingPage,
    };
}
//# sourceMappingURL=pagination.trait.selectors.js.map