import { _isNumberValue } from '@angular/cdk/coercion';
const MAX_SAFE_INTEGER = 9007199254740991;
function sortingDataAccessor(data, sortHeaderId) {
    const value = data[sortHeaderId];
    if (_isNumberValue(value)) {
        const numberValue = Number(value);
        // Numbers beyond `MAX_SAFE_INTEGER` can't be compared reliably so we
        // leave them as strings. For more info: https://goo.gl/y5vbSg
        return numberValue < MAX_SAFE_INTEGER ? numberValue : value;
    }
    return value;
}
/**
 * Gets a sorted copy of the data array based on the state of the Sort.
 * @param data The array of data that should be sorted.
 * @param sort The connected MatSort that holds the current sort state.
 */
export function sortData(data, sort) {
    const active = sort.active;
    const direction = sort.direction;
    if (!active || direction === '') {
        return data;
    }
    return data.sort((a, b) => {
        const valueA = sortingDataAccessor(a, active);
        const valueB = sortingDataAccessor(b, active);
        // If both valueA and valueB exist (truthy), then compare the two. Otherwise, check if
        // one value exists while the other doesn't. In this case, existing value should come last.
        // This avoids inconsistent results when comparing values to undefined/null.
        // If neither value exists, return 0 (equal).
        let comparatorResult = 0;
        if (valueA != null && valueB != null) {
            // Check if one value is greater than the other; if equal, comparatorResult should remain 0.
            if (typeof valueA === 'string' || typeof valueB === 'string') {
                // if either values are a string, then force both to be strings before localCompare
                comparatorResult = valueA.toString().localeCompare(valueB.toString());
            }
            else {
                if (valueA > valueB) {
                    comparatorResult = 1;
                }
                else if (valueA < valueB) {
                    comparatorResult = -1;
                }
            }
        }
        else if (valueA != null) {
            comparatorResult = 1;
        }
        else if (valueB != null) {
            comparatorResult = -1;
        }
        return comparatorResult * (direction === 'asc' ? 1 : -1);
    });
}
//# sourceMappingURL=sort.utils.js.map