import { EntityAndStatusState } from '../load-entities/load-entities.model';
import { ActionCreator, TypedAction } from '@ngrx/store/src/models';
import { KeyedConfig } from 'ngrx-traits';
export declare type SortDirection = 'asc' | 'desc' | '';
export interface Sort<T> {
    /** The id of the column being sorted. */
    active: keyof T;
    /** The sort direction. */
    direction: SortDirection;
}
export interface SortState<T> {
    sort?: {
        current: Sort<T>;
        default: Sort<T>;
    };
}
export interface EntityAndSortState<T> extends EntityAndStatusState<T>, SortState<T> {
}
export declare type SortActions<T> = {
    sort: ActionCreator<string, (props: Sort<T>) => Sort<T> & TypedAction<string>>;
    resetSort: ActionCreator<string, () => TypedAction<string>>;
};
export declare type SortSelectors<T> = {
    selectSort: (state: EntityAndSortState<T>) => Sort<T> | undefined;
};
export declare type SortMutators<T> = {
    sortEntities<S extends EntityAndSortState<T>>({ active, direction }: Sort<T>, state: S): S;
};
export declare const sortTraitKey = "sort";
export interface SortConfig<T> {
    defaultSort?: Sort<T>;
    remote?: boolean;
}
export declare type SortKeyedConfig<T> = KeyedConfig<typeof sortTraitKey, SortConfig<T>>;
